﻿define([
// dojo
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/event",
    "dojo/_base/lang",

    "dojo/dom-attr",
    "dojo/dom-class",

    "dojo/Evented",
    "dojo/mouse",
    "dojo/on",

// template
    "dojo/text!./templates/_ContentNavigationTreeNode.html",

// epi
    "epi/datetime",
    "epi/dependency",
    "epi/string",

    "epi/shell/widget/_ModelBindingMixin",
    "epi/shell/widget/Tooltip",

    "epi-cms/core/ContentReference",
    "epi-cms/widget/_ContentTreeNode",
    "epi-cms/widget/viewmodel/ContentStatusViewModel",
// resources
    "epi/i18n!epi/cms/nls/episerver.cms.components.pagetree",
    "epi/i18n!epi/cms/nls/episerver.shared.header"
],

function (
// dojo
    array,
    declare,
    event,
    lang,

    domAttr,
    domClass,

    Evented,
    mouse,
    on,

// template
    template,

// epi
    epiDate,
    dependency,
    epiString,

    _ModelBindingMixin,
    Tooltip,

    ContentReference,
    _ContentTreeNode,
    ContentStatusViewModel,
// resources
    res,
    headingResources
) {

    // summary:
    //      A customized treenode for page tree
    // tags:
    //      public

    return declare([_ContentTreeNode, _ModelBindingMixin, Evented], {
        templateString: template,
        res: res,

        // hasContextMenu: [bool]
        //      Flag used to not create context menu
        hasContextMenu: true,

        _pageLocked: 'epi-pt-accessRights',

        _pageBeingEdited: 'epi-pt-edited',

        _contextMenuClass: 'epi-iconContextMenu',

        // _iconNodeClass: [String]
        //      Custom icon for a node in the tree
        // tags:
        //      protected
        _iconNodeClass: '',

        // _iconNodeToolTip: Object
        //      Represents the tooltip for an icon node
        _iconNodeToolTip: undefined,

        _inUseTooltip: undefined,
        _selected: false,

        // _contentTypeStore: [Object]
        //      Represents the ContentType REST store to get data.
        _contentTypeStore: null,

        postMixInProperties: function () {
            this.inherited(arguments);

            var registry = dependency.resolve("epi.storeregistry");

            this._contentTypeStore = registry.get("epi.cms.contenttype");
        },

        postCreate: function () {
            this.inherited(arguments);

            this.set("model", new ContentStatusViewModel({ contentStatus: this.item.status }));
        },

        modelBindingMap: {
            "statusIcon": ["statusIcon"],
            "statusMessage": ["statusMessage"]
        },

        _setStatusMessageAttr: function (/* String */message) {
            // summary:
            //      Set content status message
            // tags:
            //      private

            domAttr.set(this.iconNodeStatus, "title", epiString.toTooltipText(message));
        },

        _setStatusIconAttr: function (/* String */cssClasses) {
            // summary:
            //      Set content status icon CSS classes
            // tags:
            //      private

            if (!(cssClasses instanceof Array)) {
                return;
            }

            //remove everything
            domClass.remove(this.iconNodeStatus);

            domClass.add(this.iconNodeStatus, "dijitTreeIcon epi-extraIcon");
            array.forEach(cssClasses, function (cssClass) {
                domClass.add(this.iconNodeStatus, cssClass);
            }, this);
        },

        destroy: function () {
            this.inherited(arguments);

            this._destroyIconNodeTooltip();
        },

        setSelected: function (/*Boolean*/selected) {
            // summary:
            //      Override to show contex menu if the node is selected
            // tags:
            //      public

            this.inherited(arguments);
            if (this.hasContextMenu) {
                domClass.toggle(this.iconNodeMenu, this._contextMenuClass, this._selected = selected);
            }
        },

        showContextMenu: function (/* Boolean */show) {
            // summary:
            //      Toogle context menu icon
            // Remark:
            //      Do not use domClass.toggle because: we show the context menu using _onNodeMouseEnter event in the Tree
            //      but this event will be trigger twice (only in IE & FireFox) once the node is updated or reloaded.
            // tags:
            //      public

            if (!this._selected && this.hasContextMenu) {
                domClass.toggle(this.iconNodeMenu, this._contextMenuClass, show);
            }
        },

        _onClickIconNodeMenu: function (evt) {
            event.stop(evt);

            // Since we're stopping the event we're preventing the regular focus handling,
            // which means that the context menu won't have anything to restore focus to when closed
            this.labelNode.focus();
            this.emit("onContextMenuClick", { node: this, target: evt.target, x: evt.pageX, y: evt.pageY });
        },

        _onMouseDownIconNodeMenu: function (evt) {
            //Stop mouse down event when clicking on the context menu icon
            event.stop(evt);
        },

        _updateIndividualLayout: function () {
            this._updateAccessRightIcon();

            if (this.model) {
                this.model.set("contentStatus", this.item.status);
            }

            // Update tooltip
            this._createTooltip();
        },

        _updateAccessRightIcon: function () {
            // summary:
            //      Set access right and icon for the tree node
            // tags:
            //      private

            // Display lock icon if current user has no permission or only has "Read" permission
            if (this.item.accessMask === 0 || this.item.accessMask == 1) {
                domClass.add(this.iconNodeAccess, this._pageLocked);
                // Add tooltip to icon
                domAttr.set(this.iconNodeAccess, "title", this.res.tooltip.pagelocked);

                // Set default background for this type of nodes
                domClass.add(this.contentNode, "epi-treeReadOnly");
            }
            else if (this.item.inUseNotifications && this.item.inUseNotifications.length > 0) {
                // Display being edited icon
                // Only display this icon if user has change permission on page
                domClass.add(this.iconNodeAccess, this._pageBeingEdited);

                // create list of user names and dates
                var tooltipRows = [{ label: this.res.tooltip.beingedited }];

                array.forEach(this.item.inUseNotifications, function (inUseNotification) {
                    var date = epiDate.toUserFriendlyHtml(inUseNotification.created);
                    tooltipRows.push({ label: inUseNotification.userName + ", " + date, htmlEncode: false });
                }, this);

                this._populateIconNodeTooltip(tooltipRows, this.iconNodeAccess);
            }
            else {
                domClass.remove(this.iconNodeAccess, this._pageBeingEdited);
                domClass.remove(this.iconNodeAccess, this._pageLocked);
                domClass.remove(this.contentNode, "epi-treeReadOnly");

                this._destroyIconNodeTooltip();
            }
        },

        _createTooltip: function () {
            // summary:
            //      Create a tooltip for the tree node and icon node.
            // tags:
            //      private

            // Don't create a tooltip if this node isn't visible.
            if(this.indent < 0) {
                return;
            }

            //We create a content reference and use the id to hide the provider key for the user.
            var reference = new ContentReference(this.item.contentLink);
            var title = this.item.name +
                ", " + headingResources.id + ": " + reference.id +
                " (" + headingResources.type + ": " + this.item.contentTypeName +")";
            domAttr.set(this.labelNode, "title", title);
            domAttr.set(this.iconNode, "title", title);
        },

        _populateIconNodeTooltip: function (/* Array */tooltipRows, /* String | DomNode */nodeToConnect) {
            // summary:
            //      Create the tooltip for target icon node
            // tags:
            //      private

            if (!this._iconNodeToolTip) {
                this._iconNodeToolTip = new Tooltip({
                    position: ["after-centered", "below"]
                });
            }
            this._iconNodeToolTip.tooltipRows = tooltipRows;

            this.own(
                on(nodeToConnect, mouse.enter, lang.hitch(this, function () {
                    if (this._iconNodeToolTip) {
                        this._iconNodeToolTip.open(nodeToConnect);
                    }
                })),
                on(nodeToConnect, mouse.leave, lang.hitch(this, function () {
                    if (this._iconNodeToolTip) {
                        this._iconNodeToolTip.close();
                    }
                }))
            );
        },

        _destroyIconNodeTooltip: function () {
            // summary:
            //      Destroy the tooltip for icon node
            // tags:
            //      private

            if (this._iconNodeToolTip) {
                this._iconNodeToolTip.destroyRecursive();
                this._iconNodeToolTip = null;
            }
        }
    });
});