﻿define([

// Dojo
    "dojo/_base/connect",
    "dojo/_base/declare",
    "dojo/_base/Deferred",
    "dojo/_base/lang",

// EPi CMS    
    "epi",
    "epi/datetime",
    "epi-cms/contentediting/ExpirationDialog",
    "epi-cms/contentediting/viewmodel/ExpirationDialogViewModel",    
    "epi-cms/contentediting/ContentActionSupport",

// EPi Framework
    "epi/dependency",
    "epi/shell/command/_Command",
    "epi/shell/widget/dialog/Dialog",

// Resources
    "epi/i18n!epi/cms/nls/episerver.cms.widget.expirationeditor",
    "epi/i18n!epi/nls/episerver.shared"
], function (

// Dojo
    connect,
    declare,
    Deferred,
    lang,

// EPi CMS    
    epi,
    epiDatetime,
    ExpirationDialog,
    ExpirationDialogViewModel,
    ContentActionSupport,

// EPi Framework
    dependency,
    _Command,
    Dialog,

// Resources
    resources,
    sharedResources
    ) {

    // module:
    //		epi/cms/contentediting/command/ManageExpiration
    // summary:
    //		Displays the manage expiration dialog for the current content.

    return declare([_Command], {

        label: resources.dialogtitle,

        tooltip: resources.dialogtitle,

        viewModel: null,

        cssClass: "epi-chromelessButton epi-visibleLink",

        constructor: function () {
            this._metadataManager = dependency.resolve("epi.shell.MetadataManager");
        },

        _execute: function () {
            // summary:
            //		Toggles the value of the given property on the model.
            // tags:
            //		protected

            if (!this._metadata) {
                return;
            }
            var viewModel = this.get("viewModel");
            var content = new ExpirationDialog({ metadata: this._metadata, value: viewModel.get("value"), model: viewModel });
            var dialog = new Dialog({
                destroyOnHide: true,
                defaultActionsVisible: false,
                confirmActionText: sharedResources.action.save,
                content: content,
                title: resources.dialogtitle
            });
            dialog.connect(dialog, "onExecute", lang.hitch(this, function () {
                this._onSave(dialog);
            }));
            dialog.connect(dialog, "onCancel", lang.hitch(this, function () {
                this._onCancel(dialog);
            }));

            dialog.show();
        },

        _onModelChange: function () {
            // summary:
            //		Updates canExecute after the model has been updated.
            // tags:
            //		protected

            var model = this.get("model");
            if (!model || !model.contentData) {
                this.set("canExecute", false);
                return;
            }

            var properties = model.contentData.properties,
                expireBlock = properties.iversionable_expire;

            this.set("viewModel", new ExpirationDialogViewModel({
                contentLink: model.contentLink,
                contentName: model.contentData.name,
                value: expireBlock,
                minimumExpireDate: properties.iversionable_startpublish
            }));

            var canExecute = model.canChangeContent() &&
                             ContentActionSupport.hasAccess(model.contentData.accessMask, ContentActionSupport.accessLevel.Publish); // Ensure the user has Publish access right

            Deferred.when(model.getPropertyMetaData("iversionable_expire"), lang.hitch(this, function (metadata) {
                this._metadata = metadata;
            }));

            this.set("canExecute", canExecute);
        },

        _onSave: function (dialog) {
            var dialogModel = dialog.content.model,
                value = dialogModel.get("intermediateValue");
            dialogModel.set("value", value);

            var model = this.get("model");            
            model.publishProperty("iversionable_expire", value);
            
        },

        _onCancel: function (dialog) {
            var dialogModel = dialog.content.model;
            dialogModel.set("intermediateValue", dialogModel.get("value"));
        }
    });
});
