﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-construct",
    "dojo/dom-style",
    "dojo/topic",
    "dojo/when",
    "dijit/_WidgetBase",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/form/Button",
    "epi/dependency",
    "dojo/text!./templates/LicenseInformation.html",
    "epi/i18n!epi/shell/ui/nls/episerver.shell.ui.resources.licenseinformation"],
function (
    declare,
    lang,
    domConstruct,
    domStyle,
    topic,
    when,
    _WidgetBase, 
    _TemplatedMixin, 
    WidgetsInTemplateMixin,
    Button,
    dependency,
    template,
    resources) {

    // module: epi/shell/widget/LicenseInformation
    // summary:
    //		Widget that displays license information from server.

    return declare([_WidgetBase, _TemplatedMixin, WidgetsInTemplateMixin], {
        // summary:
        //		Widget that displays license information from server.
        // tags:
        //		public

        // templateString: String
        //      The template string.
        templateString: template,

        // licenseInformationStore: Object
        //      The license information rest store.
        licenseInformationStore: null,

        // _res: Object
        //      I18N resources.
        _res: resources,

        postMixInProperties: function () {
            // summary:
            //      Obtains the license information store from store registry.

            this.inherited(arguments);

            var storeRegistry = dependency.resolve("epi.storeregistry");
            this.licenseInformationStore = this.licenseInformationStore || storeRegistry.get("epi.shell.licenseinformation");
        },

        buildRendering: function () {
            // summary:
            //      Build rendering from information gotten from the store.

            this.inherited(arguments);

            this.set("visible", false);

            when(this.licenseInformationStore.get(), lang.hitch(this, this._bindInformation));
        },

        _setVisibleAttr: function (visible) {
            // summary:
            //      Visible property setter
            // visible: Boolean
            //      Indicates that the widget is visible or not.

            domStyle.set(this.domNode, "display", visible ? "" : "none");
        },

        _onRefreshButtonClick: function () {
            // summary:
            //      Handles the Refresh button.

            when(this.licenseInformationStore.refresh('1'), lang.hitch(this, this._bindInformation));
        },

        _bindInformation: function (licenseInformation) {
            // summary:
            //      Binds license information to the widget.

            if (!licenseInformation || licenseInformation.isValid) {
                this.set("visible", false);
            } else {
                this.set("visible", true);
                this.mainInfoNode.innerHTML = licenseInformation.message;
                this.additionalInfoNode.innerHTML = licenseInformation.additionalInformation;
            }

            // Tells the root container to re-layout
            topic.publish("/epi/shell/globalnavigation/layoutchange");
        }
    });
});